<!DOCTYPE html>
<html lang="en">
  <head>
    <title>{% block title %}{{ title }}{% endblock %}</title>

    {% block head %}{% endblock %}

    {% if favicon_url %}
      <link rel="shortcut icon" href="{{ favicon_url }}" />
    {% endif %}

    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, height=device-height,
                                   initial-scale=1.0, maximum-scale=1.0, user-scalable=no, minimal-ui"/>
  </head>
  <body>
    {% block content %}{% endblock %}
  </body>
</html>
